@@ -151,3 +151,31 @@ test_that("CYCLE=DROP",{
     nm1 <- NMinfo(res)
     expect_equal_to_reference(nm1,fileRef,version=2)
 })
+
+
+
+test_that("No filters",{
+
+    fileRef <- "testReference/NMscanInput_08.rds"
+
+    inp <- NMscanInput("testData/nonmem/xgxr027.lst")
+    inp <- fix.time(inp)
+
+    expect_equal_to_reference(inp,fileRef,version=2)
+})
+
+
+test_that("Multiple filters on same column",{
+    NMdataConf(as.fun="data.table")
+    fileRef <- "testReference/NMscanInput_09.rds"
+
+    inp.nofilt <- NMscanInput("testData/nonmem/xgxr029.mod",applyFilters=FALSE)[,data:="nofilt"]
+    inp.filt <- NMscanInput("testData/nonmem/xgxr029.mod",applyFilters=TRUE)[,data:="filt"]
+    inp <- rbind(inp.nofilt,inp.filt)
+    
+    tab.count <- dcast(
+        inp[,.N,by=.(ID,data)]
+       ,ID~data,value.var="N")
+
+    expect_equal_to_reference(tab.count,fileRef,version=2)
+})
