@@ -15,6 +15,7 @@ test_that("basic",{
     mc1 <- mergeCheck(df1,df2,by="y")
     
     expect_equal_to_reference(mc1,fileRef,version=2)
+
 })
 
 
@@ -123,7 +124,9 @@ test_that("Zero-row df1 must give an error",{
     dt3 <- dt2[,!c("b")]
 
     ## compareCols(dt1,dt3,diff.only=FALSE)
-    expect_error(mergeCheck(dt1,dt3,by="a"))
+    expect_error(
+        mergeCheck(dt1,dt3,by="a")
+    )
 
 })
 
@@ -134,7 +137,42 @@ test_that("Duplicating input rows",{
 
     ## compareCols(dt1,dt3,diff.only=FALSE)
     expect_error(
-            mergeCheck(dt1,dt3,by="a",ncols.expect = 1)
+        mergeCheck(dt1,dt3,by="a",ncols.expect = 1)
+    )
+
+})
+
+test_that("deprecated df1 and df2",{
+    dt1 <- data.table(x = 1:10,
+                      y=letters[1:10])
+    df2 <- data.frame(y=letters[1:11],
+                      x2 = 1:11,
+                      stringsAsFactors=FALSE)
+
+    ## compareCols(dt1,dt3,diff.only=FALSE)
+    expect_message(
+        mergeCheck(df1=dt1,df2=df2,by="y",ncols.expect = 1)
+    )
+
+
+})
+
+test_that("missing values in by",{
+    dt1 <- data.table(x = 1:10,
+                      y=c(letters[1:9],NA))
+    dt2 <- data.table(y=letters[1:11],
+                      x2 = 1:11)
+
+    ## compareCols(dt1,dt3,diff.only=FALSE)
+    ##
+    ## mergeCheck(dt1,dt2,by="y",ncols.expect = 1)
+    ## mergeCheck(dt1,dt2,by="y",ncols.expect = 1,all.x=T)
+    dt2[3,y:=NA]
+    expect_error(
+        mergeCheck(dt1,dt2,by="y",ncols.expect = 1)
+    )
+    expect_error(
+        mergeCheck(dt2,dt1,by="y",ncols.expect = 1)
     )
 
 })
